/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module time_data(
    clk        ,
    rst_n      ,
    key_vld    ,
    end_counter,
    xs_sec_low ,
    xs_sec_high,
    xs_min_low ,
    xs_min_high,
    sec_low    ,
    sec_high   ,
    min_low    ,
    min_high   ,
    flag_set
);
input           clk             ;
input           rst_n           ;
input   [3:0 ]  key_vld         ;
output          end_counter     ;
output  [3:0 ]  xs_sec_low      ;
output  [3:0 ]  xs_sec_high     ;
output  [3:0 ]  xs_min_low      ;
output  [3:0 ]  xs_min_high     ;
output  [3:0 ]  sec_low         ;
output  [3:0 ]  sec_high        ;
output  [3:0 ]  min_low         ;
output  [3:0 ]  min_high        ;
output          flag_set        ;
reg             flag_set        ;
reg     [2:0 ]  sel_cnt         ;
wire            add_sel_cnt     ;
wire            end_sel_cnt     ;
reg             set_en          ;
reg     [25:0]  counter         ;
reg     [3:0 ]  sec_low         ;
reg     [3:0 ]  sec_high        ;
reg     [3:0 ]  min_low         ;
reg     [3:0 ]  min_high        ;
reg             flag            ;
reg     [3:0 ]  xs_sec_low      ;
reg     [3:0 ]  xs_sec_high     ;
reg     [3:0 ]  xs_min_low      ;
reg     [3:0 ]  xs_min_high     ;
reg             init            ;
wire            add_miao        ;
wire            end_miao        ;
wire            add_xs_min_high ;
wire            end_xs_min_high ;
wire            add_xs_min_low  ;
wire            end_xs_min_low  ;
wire            add_xs_sec_high ;
wire            end_xs_sec_high ;
wire            add_xs_sec_low  ;
wire            end_xs_sec_low  ;
wire            add_min_high    ;
wire            end_min_high    ;
wire            add_min_low     ;
wire            end_min_low     ;
wire            add_sec_high    ;
wire            end_sec_high    ;
wire            add_sec_low     ;
wire            end_sec_low     ;
wire            add_counter     ;
wire            end_counter     ;



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        flag_set<=1'b0;
    end
    else if(key_vld[0]==1'b1)begin
        flag_set<=~flag_set;
    end
    else begin
        flag_set<=flag_set;
    end
end


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        sel_cnt <= 0; 
    end
    else if(add_sel_cnt) begin
        if(end_sel_cnt)
            sel_cnt <= 0; 
        else
            sel_cnt <= sel_cnt+1 ;
   end
end
assign add_sel_cnt = key_vld[1]==1'b1;
assign end_sel_cnt = add_sel_cnt  && sel_cnt == 8-1 ;




always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        set_en<=1'b0;
    end
    else if(flag_set==1'b1 && key_vld[2]==1'b1)begin
        set_en<=1'b1;
    end
    else begin
        set_en<=1'b0;
    end
end


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        counter <= 0; 
    end
    else if(add_counter) begin
        if(end_counter)
            counter <= 0; 
        else
            counter <= counter+1 ;
   end
end
assign add_counter = flag_set==1'b0;
assign end_counter = add_counter  && counter == 26'd5000_0000-1;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        sec_low <= 0; 
    end
    else if(add_sec_low) begin
        if(end_sec_low)
            sec_low <= 0; 
        else
            sec_low <= sec_low+1 ;
   end
end
assign add_sec_low = sel_cnt==1-1 && set_en;
assign end_sec_low = add_sec_low  && sec_low == 10-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        sec_high <= 0; 
    end
    else if(add_sec_high) begin
        if(end_sec_high)
            sec_high <= 0; 
        else
            sec_high <= sec_high+1 ;
   end
end
assign add_sec_high = sel_cnt==2-1 && set_en;
assign end_sec_high = add_sec_high  && sec_high == 6-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        min_low <= 0; 
    end
    else if(add_min_low) begin
        if(end_min_low)
            min_low <= 0; 
        else
            min_low <= min_low+1 ;
   end
end
assign add_min_low = sel_cnt==3-1 && set_en;
assign end_min_low = add_min_low  && min_low == 10-1 ;

always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        min_high <= 0; 
    end
    else if(add_min_high) begin
        if(end_min_high)
            min_high <= 0; 
        else
            min_high <= min_high+1 ;
   end
end
assign add_min_high = sel_cnt==4-1 && set_en;
assign end_min_high = add_min_high  && min_high == 6-1 ;




always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        xs_sec_low <= 0; 
    end
    else if(add_xs_sec_low) begin
        if(end_xs_sec_low)
            xs_sec_low <= 0; 
        else
            xs_sec_low <= xs_sec_low+1 ;
   end
end
assign add_xs_sec_low = (sel_cnt==5-1 && set_en) || end_counter;
assign end_xs_sec_low = add_xs_sec_low && xs_sec_low == 10-1 ;




always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        xs_sec_high <= 0; 
    end
    else if(add_xs_sec_high) begin
        if(end_xs_sec_high)
            xs_sec_high <= 0; 
        else
            xs_sec_high <= xs_sec_high+1 ;
   end
end
assign add_xs_sec_high = (sel_cnt==6-1 && set_en) || end_xs_sec_low;
assign end_xs_sec_high = add_xs_sec_high  && xs_sec_high == 6-1 ;



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        xs_min_low <= 0; 
    end
    else if(add_xs_min_low) begin
        if(end_xs_min_low)
            xs_min_low <= 0; 
        else
            xs_min_low <= xs_min_low+1 ;
   end
end
assign add_xs_min_low = (sel_cnt==7-1 && set_en) || end_xs_sec_high;
assign end_xs_min_low = add_xs_min_low  && xs_min_low == 10-1 ;




always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        xs_min_high <= 0; 
    end
    else if(add_xs_min_high) begin
        if(end_xs_min_high)
            xs_min_high <= 0; 
        else
            xs_min_high <= xs_min_high+1 ;
   end
end
assign add_xs_min_high = (sel_cnt==8-1 && set_en) || end_xs_min_low;
assign end_xs_min_high = add_xs_min_high  && xs_min_high == 6-1 ;




endmodule
